const Splitty = (function(){
        var actions = [];
        var self = {};
        var configuration = null;
        var aes_key = "";
        var _socket = null;
        var _editor = null;
        self.setKey   = function(key) { aes_key = key };
        self.register = function(module) { actions.push(module) };
        self.startup  = function()  {
            executeActions();
            openUserDefinedFiles();
            setupUserDefinedAliases();
            setupAceEditorOptions();
        }
        function setupAceEditorOptions(){
            if(configuration["editor"] && configuration["editor"]["ace"] && configuration["editor"]["ace"]["editorOptions"]){
                _editor.setOptions(configuration["editor"]["ace"]["editorOptions"]);
            }
        }
        function executeActions(){
            actions.each(function(module){
                if(typeof module["startup"] === "function"){
                    module.startup()
                }
            });
        };
        function setupUserDefinedAliases(){
            if(configuration["editor"] && configuration["editor"]["aliases"]){
                for(var alias in configuration["editor"]["aliases"]){
                    if(!self.hasAlias(alias)){
                        self.addAlias(alias,configuration["editor"]["aliases"][alias]);
                    }
                };
            }
        }
        function openUserDefinedFiles(){
            if(!_socket)throw "Socket not defined to Splitty Class, please configure socket object by using setSocket method from Splitty class.";
            if(configuration["files"] && configuration["files"]["toOpen"]){
                configuration["files"]["toOpen"].each(function(fileName) {
                    FileAction.touch(fileName, function(){
                        FileAction.openSelectedFile(fileName);    
                    })
                });
                self.emit("deleteConfigSections",["files"]);
            }
        }
        
        
        
        const  transformToAssocArray = function( prmstr ) {
            var params = {};
            var prmarr = prmstr.split("&");
            for ( var i = 0; i < prmarr.length; i++) {
                var tmparr = prmarr[i].split("=");
                params[tmparr[0]] = tmparr[1];
            }
            return params;
        };
        
        self.params = function()  {
            var prmstr = window.parent.location.search.substr(1);
            return prmstr !== null && prmstr !== "" ? transformToAssocArray(prmstr) : {};
        };
        
        self.prop = function(prop,value)  {
            if(typeof(value) !== "undefined"){
                localStorage["$$splitty$$"+configuration.host_path+"$$"+prop] = value;
            }else return localStorage["$$splitty$$"+configuration.host_path+"$$"+prop];
            
        };
        self.global = function(prop,value)  {
            if(typeof(value) !== "undefined"){
                localStorage["$$splitty$$"+prop] = value;
            }else return localStorage["$$splitty$$"+prop];
            
        };
        self.hasProp = function(prop) { 
            return typeof(localStorage["$$splitty$$"+configuration.host_path+"$$"+prop]) !== "undefined";
        }; 
        
        self.hasGlobal = function(prop) {
            return typeof(localStorage["$$splitty$$"+prop]) !== "undefined";
        };
        
        self.setConfig = function(config) { configuration = config };
        
        self.config = function() { return  configuration };
        
        self.encrypt = function(message)  {
            if(typeof(message) !== "string"){
                message = JSON.stringify(message);   
            }
            return CryptoJS.AES.encrypt(message, aes_key).toString();
        };
        
        self.decrypt = function(message)  {
            var bytes  = CryptoJS.AES.decrypt(message, aes_key);
            var plaintext = bytes.toString(CryptoJS.enc.Utf8);
            try{
                return JSON.parse(plaintext);
            }catch (e){
                return plaintext;
            }
        };
        
        self.setSocket = function(socket) { _socket = socket };
        
        self.setEditor = function(editor) { _editor = editor };
        
        self.setup = function(){
            var params = self.params();
            self.setKey(params["key"]);
        };
        
        self.isWindows = function(){
        	return configuration.platform.startsWith("win");
        };

        self.delimiter = function(){
            //if(self.isWindows()){
            //    return "\\";
            //}
            return "/"
        }

        self.emit = function(channel,data)  {
            _socket.emit(channel,self.encrypt(data));
        };
        
        self.receive = function(channel,callback)  {
            _socket.on(channel,function(data){
               var plainData = self.decrypt(data); 
               callback(plainData);
            });
        };
        self.addAlias = function(alias, command)  {
            self.global("alias$$"+alias,command);
        };
        
        self.getAlias = function(alias)  {
            return self.global("alias$$"+alias);
        };
        
        self.hasAlias = function(alias)  {
            return self.hasGlobal("alias$$"+alias)
        };
        self.isUnix = function(){
        	return !self.isWindows();
        }
        

        self.setOnSession = function(key,value){
            sessionStorage[key] = JSON.stringify(value);
        };

        self.getOnSession = function(key){
            var data = sessionStorage[key];
            if (data) {
                return JSON.parse(data);
            }
        };

        self.deleteOnSession = function(key){
            delete sessionStorage[key];
        };

        self.join = function(tail,head){
            if(self.isUnix()){
                return tail + "/" + head;
            }else{
                return tail + "\\" + head;
            }
        };

        self.getFileName = function(path){
            if(path.indexOf("/") >= 0){
                return path.split("/").last();
            }else{
                return path.split("\\").last();
            }
        };

        self.onUnix = function(unixExec,otherExec){
            if (self.isUnix() && typeof(unixExec) === "function"){
                unixExec();
            }else if (typeof(otherExec) === "function"){
                otherExec();
            }
        };

        return self;
    })();


    
