#! /usr/bin/env node

verifyNodeVersion();

var http         = require('http');
var finalhandler = require('finalhandler');
var serveStatic  = require('serve-static');
var request      = require('request');
var serve        = serveStatic("./");
var editor       = serveStatic(__dirname);
var fs           = require("fs");
var sys          = require('sys');
const spawn      = require('child_process').spawn;
const cache      = require('./app/cache/cache');
var open         = require('open');

var crypt        = require('./app/security/crypt'); 
require('shelljs/global');
var config = require('./app/conf/splitty');
var server = http.createServer((req, res) => {
  var done = finalhandler(req, res);
  if(isWebEditor(req)){      
    editor(req,res,done);
  }else if(isHelp(req)){
    editor(req, res, done);
  }else if(isTerminal(req)){
    editor(req,res,done);
  }else if(isSplitty(req,res,done)){
      splitty(req,res,done);
  }
  else{
    serve(req, res, done);
  }
});
require('./app/io/sock').setServer(server);

function isOSX(){
    return config.platform === "darwin";
}
function isWindows(){
    return config.platform.startsWith("win");
}
function isLinux(){
    return config.platform === "linux";
}
function delimiter(){
    if(isWindows()){
        return "\\";
    }
    return "/";
}
function verifyNodeVersion(){
    var version = process.version;
    version     = version.replace("v","");
    var parts   = version.split(".");
    var majorVersion  = parseInt(parts[0]);
    var middleVersion = parseInt(parts[1]);
    var minorVersion  = parseInt(parts[2]);
    if(majorVersion < 4){
        console.log("Your current node version is not supported!");
        console.log("node version: " + process.version);
        console.log("Please, update you node to 4+ version");
        process.exit();
    }
};

function listen(port) {
  console.log("PID: " + process.pid);
  server.on("error",(err)=>{
      config.port = port+1;
      userArgs.push("port="+(config.port));
      var instance = spawn("splitty",userArgs);
      instance.stdout.on('data', (data) => {
        console.log(data.toString("utf-8"));
      });
  });
  server.listen(port, config["bindAddress"],()=>{    
    console.log("To open splitty editor browse to");
    var splitty_url = "http://localhost:"+config.port+"/_editor?key="+config.key+"&proportion=0";
    console.log(splitty_url);
    open(splitty_url);
  });
};
function isWebEditor(request){
    return isArea(request,"_editor");
}
function isHelp(request){        
    return isArea(request,"_help");
}
function isTerminal(request){
   return isArea(request,"_terminal");
}

function isArea(request,area){
    var url = require('url');    
    var url_parts = url.parse(request.url, true);
    return url_parts.pathname.indexOf(area) > 0;
}

function isSplitty(request){
   return isArea(request,"_splitty");
}





String.prototype.replaceAll = function(search, replacement) {
    var target = this;
    return target.split(search).join(replacement);
};

listen(parseInt(config["port"]));

var controllers  = require('./routes');

function splitty(req,res,done){
    res.statusCode = 200;
    res.setHeader('Content-Type', 'application/json');    
    var url = req.url.split("?")[0];     
    if(typeof(controllers.routes[req.method+url]) !== "undefined"){
        controllers.routes[req.method+url](req,res,done);
    }else{
        res.end("url not found\n");
    }  
}

global.isWindows = isWindows;
global.delimiter = delimiter;

